# Copyright (C) 2022 CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

# CVAT REST API
#
# REST API for Computer Vision Annotation Tool (CVAT)  # noqa: E501
#
# The version of the OpenAPI document: 2.1.0
# Contact: support@cvat.ai
# Generated by: https://openapi-generator.tech


# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from cvat_sdk.api_client.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from cvat_sdk.api_client.model.about import About, IAbout
from cvat_sdk.api_client.model.annotation_file_request import AnnotationFileRequest, IAnnotationFileRequest
from cvat_sdk.api_client.model.annotations_read import AnnotationsRead
from cvat_sdk.api_client.model.attribute import Attribute, IAttribute
from cvat_sdk.api_client.model.attribute_request import AttributeRequest, IAttributeRequest
from cvat_sdk.api_client.model.attribute_val import AttributeVal, IAttributeVal
from cvat_sdk.api_client.model.attribute_val_request import AttributeValRequest, IAttributeValRequest
from cvat_sdk.api_client.model.backup_write_request import BackupWriteRequest
from cvat_sdk.api_client.model.basic_user import BasicUser, IBasicUser
from cvat_sdk.api_client.model.basic_user_request import BasicUserRequest, IBasicUserRequest
from cvat_sdk.api_client.model.chunk_type import ChunkType, IChunkType
from cvat_sdk.api_client.model.cloud_storage_read import CloudStorageRead, ICloudStorageRead
from cvat_sdk.api_client.model.cloud_storage_write_request import CloudStorageWriteRequest, ICloudStorageWriteRequest
from cvat_sdk.api_client.model.comment_read import CommentRead, ICommentRead
from cvat_sdk.api_client.model.comment_read_owner import CommentReadOwner
from cvat_sdk.api_client.model.comment_write_request import CommentWriteRequest, ICommentWriteRequest
from cvat_sdk.api_client.model.credentials_type_enum import CredentialsTypeEnum, ICredentialsTypeEnum
from cvat_sdk.api_client.model.data_meta_read import DataMetaRead, IDataMetaRead
from cvat_sdk.api_client.model.data_request import DataRequest, IDataRequest
from cvat_sdk.api_client.model.dataset_file_request import DatasetFileRequest, IDatasetFileRequest
from cvat_sdk.api_client.model.dataset_format import DatasetFormat, IDatasetFormat
from cvat_sdk.api_client.model.dataset_formats import DatasetFormats, IDatasetFormats
from cvat_sdk.api_client.model.dataset_write_request import DatasetWriteRequest
from cvat_sdk.api_client.model.exception import Exception, IException
from cvat_sdk.api_client.model.exception_request import ExceptionRequest, IExceptionRequest
from cvat_sdk.api_client.model.file_info import FileInfo, IFileInfo
from cvat_sdk.api_client.model.file_info_type_enum import FileInfoTypeEnum, IFileInfoTypeEnum
from cvat_sdk.api_client.model.frame_meta import FrameMeta, IFrameMeta
from cvat_sdk.api_client.model.input_type_enum import InputTypeEnum, IInputTypeEnum
from cvat_sdk.api_client.model.invitation_read import InvitationRead, IInvitationRead
from cvat_sdk.api_client.model.invitation_write import InvitationWrite, IInvitationWrite
from cvat_sdk.api_client.model.invitation_write_request import InvitationWriteRequest, IInvitationWriteRequest
from cvat_sdk.api_client.model.issue_read import IssueRead, IIssueRead
from cvat_sdk.api_client.model.issue_write_request import IssueWriteRequest, IIssueWriteRequest
from cvat_sdk.api_client.model.job_annotations_update_request import JobAnnotationsUpdateRequest
from cvat_sdk.api_client.model.job_commit import JobCommit, IJobCommit
from cvat_sdk.api_client.model.job_read import JobRead, IJobRead
from cvat_sdk.api_client.model.job_stage import JobStage, IJobStage
from cvat_sdk.api_client.model.job_status import JobStatus, IJobStatus
from cvat_sdk.api_client.model.label import Label, ILabel
from cvat_sdk.api_client.model.labeled_data import LabeledData, ILabeledData
from cvat_sdk.api_client.model.labeled_data_request import LabeledDataRequest, ILabeledDataRequest
from cvat_sdk.api_client.model.labeled_image import LabeledImage, ILabeledImage
from cvat_sdk.api_client.model.labeled_image_request import LabeledImageRequest, ILabeledImageRequest
from cvat_sdk.api_client.model.labeled_shape import LabeledShape, ILabeledShape
from cvat_sdk.api_client.model.labeled_shape_request import LabeledShapeRequest, ILabeledShapeRequest
from cvat_sdk.api_client.model.labeled_track import LabeledTrack, ILabeledTrack
from cvat_sdk.api_client.model.labeled_track_request import LabeledTrackRequest, ILabeledTrackRequest
from cvat_sdk.api_client.model.location_enum import LocationEnum, ILocationEnum
from cvat_sdk.api_client.model.log_event import LogEvent, ILogEvent
from cvat_sdk.api_client.model.log_event_request import LogEventRequest, ILogEventRequest
from cvat_sdk.api_client.model.login_request import LoginRequest, ILoginRequest
from cvat_sdk.api_client.model.manifest import Manifest, IManifest
from cvat_sdk.api_client.model.manifest_request import ManifestRequest, IManifestRequest
from cvat_sdk.api_client.model.membership_read import MembershipRead, IMembershipRead
from cvat_sdk.api_client.model.membership_write import MembershipWrite, IMembershipWrite
from cvat_sdk.api_client.model.meta_user import MetaUser
from cvat_sdk.api_client.model.operation_status import OperationStatus, IOperationStatus
from cvat_sdk.api_client.model.organization_read import OrganizationRead, IOrganizationRead
from cvat_sdk.api_client.model.organization_write import OrganizationWrite, IOrganizationWrite
from cvat_sdk.api_client.model.organization_write_request import OrganizationWriteRequest, IOrganizationWriteRequest
from cvat_sdk.api_client.model.paginated_cloud_storage_read_list import PaginatedCloudStorageReadList, IPaginatedCloudStorageReadList
from cvat_sdk.api_client.model.paginated_comment_read_list import PaginatedCommentReadList, IPaginatedCommentReadList
from cvat_sdk.api_client.model.paginated_invitation_read_list import PaginatedInvitationReadList, IPaginatedInvitationReadList
from cvat_sdk.api_client.model.paginated_issue_read_list import PaginatedIssueReadList, IPaginatedIssueReadList
from cvat_sdk.api_client.model.paginated_job_commit_list import PaginatedJobCommitList, IPaginatedJobCommitList
from cvat_sdk.api_client.model.paginated_job_read_list import PaginatedJobReadList, IPaginatedJobReadList
from cvat_sdk.api_client.model.paginated_membership_read_list import PaginatedMembershipReadList, IPaginatedMembershipReadList
from cvat_sdk.api_client.model.paginated_meta_user_list import PaginatedMetaUserList, IPaginatedMetaUserList
from cvat_sdk.api_client.model.paginated_polymorphic_project_list import PaginatedPolymorphicProjectList, IPaginatedPolymorphicProjectList
from cvat_sdk.api_client.model.paginated_task_read_list import PaginatedTaskReadList, IPaginatedTaskReadList
from cvat_sdk.api_client.model.password_change_request import PasswordChangeRequest, IPasswordChangeRequest
from cvat_sdk.api_client.model.password_reset_confirm_request import PasswordResetConfirmRequest, IPasswordResetConfirmRequest
from cvat_sdk.api_client.model.password_reset_serializer_ex_request import PasswordResetSerializerExRequest, IPasswordResetSerializerExRequest
from cvat_sdk.api_client.model.patched_cloud_storage_write_request import PatchedCloudStorageWriteRequest, IPatchedCloudStorageWriteRequest
from cvat_sdk.api_client.model.patched_comment_write_request import PatchedCommentWriteRequest, IPatchedCommentWriteRequest
from cvat_sdk.api_client.model.patched_data_meta_write_request import PatchedDataMetaWriteRequest, IPatchedDataMetaWriteRequest
from cvat_sdk.api_client.model.patched_invitation_write_request import PatchedInvitationWriteRequest, IPatchedInvitationWriteRequest
from cvat_sdk.api_client.model.patched_issue_write_request import PatchedIssueWriteRequest, IPatchedIssueWriteRequest
from cvat_sdk.api_client.model.patched_job_write_request import PatchedJobWriteRequest, IPatchedJobWriteRequest
from cvat_sdk.api_client.model.patched_label_request import PatchedLabelRequest, IPatchedLabelRequest
from cvat_sdk.api_client.model.patched_labeled_data_request import PatchedLabeledDataRequest, IPatchedLabeledDataRequest
from cvat_sdk.api_client.model.patched_membership_write_request import PatchedMembershipWriteRequest, IPatchedMembershipWriteRequest
from cvat_sdk.api_client.model.patched_organization_write_request import PatchedOrganizationWriteRequest, IPatchedOrganizationWriteRequest
from cvat_sdk.api_client.model.patched_project_write_request import PatchedProjectWriteRequest, IPatchedProjectWriteRequest
from cvat_sdk.api_client.model.patched_project_write_request_target_storage import PatchedProjectWriteRequestTargetStorage
from cvat_sdk.api_client.model.patched_task_write_request import PatchedTaskWriteRequest, IPatchedTaskWriteRequest
from cvat_sdk.api_client.model.patched_task_write_request_target_storage import PatchedTaskWriteRequestTargetStorage
from cvat_sdk.api_client.model.patched_user_request import PatchedUserRequest, IPatchedUserRequest
from cvat_sdk.api_client.model.plugins import Plugins, IPlugins
from cvat_sdk.api_client.model.polymorphic_project import PolymorphicProject
from cvat_sdk.api_client.model.project_file_request import ProjectFileRequest, IProjectFileRequest
from cvat_sdk.api_client.model.project_read import ProjectRead, IProjectRead
from cvat_sdk.api_client.model.project_read_assignee import ProjectReadAssignee
from cvat_sdk.api_client.model.project_read_owner import ProjectReadOwner
from cvat_sdk.api_client.model.project_read_target_storage import ProjectReadTargetStorage
from cvat_sdk.api_client.model.project_search import ProjectSearch, IProjectSearch
from cvat_sdk.api_client.model.project_write_request import ProjectWriteRequest, IProjectWriteRequest
from cvat_sdk.api_client.model.provider_type_enum import ProviderTypeEnum, IProviderTypeEnum
from cvat_sdk.api_client.model.rest_auth_detail import RestAuthDetail, IRestAuthDetail
from cvat_sdk.api_client.model.restricted_register import RestrictedRegister, IRestrictedRegister
from cvat_sdk.api_client.model.restricted_register_request import RestrictedRegisterRequest, IRestrictedRegisterRequest
from cvat_sdk.api_client.model.role_enum import RoleEnum, IRoleEnum
from cvat_sdk.api_client.model.rq_status import RqStatus, IRqStatus
from cvat_sdk.api_client.model.rq_status_state_enum import RqStatusStateEnum, IRqStatusStateEnum
from cvat_sdk.api_client.model.segment import Segment, ISegment
from cvat_sdk.api_client.model.shape_type import ShapeType, IShapeType
from cvat_sdk.api_client.model.signing_request import SigningRequest, ISigningRequest
from cvat_sdk.api_client.model.simple_job import SimpleJob, ISimpleJob
from cvat_sdk.api_client.model.sorting_method import SortingMethod, ISortingMethod
from cvat_sdk.api_client.model.storage import Storage, IStorage
from cvat_sdk.api_client.model.storage_method import StorageMethod, IStorageMethod
from cvat_sdk.api_client.model.storage_request import StorageRequest, IStorageRequest
from cvat_sdk.api_client.model.storage_type import StorageType, IStorageType
from cvat_sdk.api_client.model.sub_labeled_shape import SubLabeledShape, ISubLabeledShape
from cvat_sdk.api_client.model.sub_labeled_shape_request import SubLabeledShapeRequest, ISubLabeledShapeRequest
from cvat_sdk.api_client.model.sub_labeled_track import SubLabeledTrack, ISubLabeledTrack
from cvat_sdk.api_client.model.sub_labeled_track_request import SubLabeledTrackRequest, ISubLabeledTrackRequest
from cvat_sdk.api_client.model.sublabel import Sublabel, ISublabel
from cvat_sdk.api_client.model.sublabel_request import SublabelRequest, ISublabelRequest
from cvat_sdk.api_client.model.task_annotations_update_request import TaskAnnotationsUpdateRequest
from cvat_sdk.api_client.model.task_annotations_write_request import TaskAnnotationsWriteRequest
from cvat_sdk.api_client.model.task_file_request import TaskFileRequest, ITaskFileRequest
from cvat_sdk.api_client.model.task_read import TaskRead, ITaskRead
from cvat_sdk.api_client.model.task_read_target_storage import TaskReadTargetStorage
from cvat_sdk.api_client.model.task_write_request import TaskWriteRequest, ITaskWriteRequest
from cvat_sdk.api_client.model.token import Token, IToken
from cvat_sdk.api_client.model.tracked_shape import TrackedShape, ITrackedShape
from cvat_sdk.api_client.model.tracked_shape_request import TrackedShapeRequest, ITrackedShapeRequest
from cvat_sdk.api_client.model.user import User, IUser
from cvat_sdk.api_client.model.user_agreement import UserAgreement, IUserAgreement
from cvat_sdk.api_client.model.user_agreement_request import UserAgreementRequest, IUserAgreementRequest
